# LLM's

[Here](overview.md) is an overview of the SOTA model E2E training.

## Architecture
- ["Retentive Network: A Successor to Transformer for Large Language Models"
  (aug. 2023) - Microsoft Research](https://arxiv.org/abs/2307.08621) -
  introduces the retention mechanism to substitute attention in Transformers;
  there is a decent overview of the paper as a Medium post
  [here](https://medium.com/ai-fusion-labs/retentive-networks-retnet-explained-the-much-awaited-transformers-killer-is-here-6c17e3e8add8)
- ["Simplifying Transformer Blocks" (nov. 2023) - He et
  al.](https://arxiv.org/abs/2311.01906)
- ["BitNet: Scaling 1-bit Transformers for Large Language Models" (Wang et al. -
  oct. 2023)](https://arxiv.org/abs/2310.11453)

## Quantization

## Prompt engineering
- ["Language models are few-shot learners" (Brown et al. (OpenAI) - may.
  2020)](https://arxiv.org/abs/2005.14165) - introduces the GPT-3 (175B
  parameters) model and the technique of prompt engineering by using few-shot
  learning
- ["Chain-of-Thought Prompting Elicits Reasoning in Large Language Models" (Wei
  et al. - jan. 2022)](https://arxiv.org/abs/2201.11903) - identifies
  chain-of-thought as a very helpful prompt engineering technique, which further
  increases a model's performance on a number of benchmarked tasks

## Models
- ["Scaling Language Models: Methods, Analysis & Insights from Training Gopher"
  (Rae et al. - dec. 2021)](https://arxiv.org/abs/2112.11446) - Gopher 280B
- ["Training compute-optimal Large Language Models" (Hoffmann et al. - mar.
  2022)](https://arxiv.org/abs/2203.15556) - Chinchilla 70B
- ["PaLM: Scaling Language Modeling with Pathways" (Chowdhery et al. - apr.
  2022)](https://arxiv.org/abs/2204.02311) - Google PaLM
- ["LLaMA: Open and Efficient Foundation Language Models" (Touvron et al. - feb.
  2023)](https://arxiv.org/abs/2302.13971) - LLaMA
- ["GPT-4 Technical Report" (OpenAI - mar.
  2023)](https://arxiv.org/abs/2303.08774) - GPT-4
- ["PaLM 2 Technical Report" (Anil et al. - may
  2023)](https://arxiv.org/abs/2305.10403) - Google PaLM 2
- ["Llama 2: Open Foundation and Fine-Tuned Chat Models" (Touvron et al. - jul.
  2023)](https://arxiv.org/abs/2307.09288) - LLaMA 2

## Fine-tuning
- ["LoRA: Low-Rank Adaptation of Large Language Models" (Hu et al. - jun.
  2021)](https://arxiv.org/abs/2106.09685) - introduces fine-tuning using the
  LoRA algorithm, which helps train models for downstream tasks faster
- ["QLoRA: Efficient Finetuning of Quantized LLMs" (Dettmers et al. - may
  2023)](https://arxiv.org/abs/2305.14314) - builds on top of LoRA, and further
  quantizes models to 4-bit, to reduce memory usage

## Tokenization

- ["Neural Machine Translation of Rare Words with Subword Units" (Sennrich et
  al. - jun. 2016)](https://arxiv.org/abs/1508.07909) - introduces BPE
  (Byte-Pair Encoding), as a solution for encoding subword units
- ["Subword Regularization: Improving Neural Network Translation Models with
  Multiple Subword Candidates" (Kudo - apr.
  2018)](https://arxiv.org/abs/1804.10959) - the unigram language model, as an
  improvement over BPE; both are used today, though
- ["SentencePiece: A simple and language independent subword tokenizer and
  detokenizer for Neural Text Processing" (Kudo et al. - aug.
  2018)](https://arxiv.org/abs/1808.06226) - Google's SentencePiece tokenizer,
  a tokenization method with API's in C++ and Python, which can use either
  unigrams or BPE as its encoding method

## Mixed precision training
- ["Mixed Precision Training" (Micikevicious et al. (Nvidia) - oct.
  2017)](https://arxiv.org/abs/1710.03740) - introduces mixed precision training
- ["Mixed-Precision Training of Deep Neural Networks"
  (Nvidia)](https://developer.nvidia.com/blog/mixed-precision-training-deep-neural-networks/)
  - 2017 Nvidia blogpost (accompanying the paper above) about mixed-precision
    training, implemented on Nvidia GPU's
- ["Mixed-Precision Training of Deep Neural Networks" video
  (Nvidia)](https://on-demand.gputechconf.com/gtc/2019/video/_/S9143/) - the
  video accompanying the above blogpost/paper
- ["Train with Mixed Precision"
  (Nvidia)](https://docs.nvidia.com/deeplearning/performance/mixed-precision-training/index.html)
  - documentation on training models with Nvidia GPU's using mixed precision
- ["What Every User Should Know About Mixed Precision Training in PyTorch"
  (PyTorch)](https://pytorch.org/blog/what-every-user-should-know-about-mixed-precision-training-in-pytorch/)
  - intro to mixed precision in PyTorch
- ["Automatic Mixed Precision"
  (PyTorch)](https://pytorch.org/tutorials/recipes/recipes/amp_recipe.html) - a
  short recipe on how to add mixed precision in PyTorch
- ["CUDA Automatic Mixed Precision examples"
  (PyTorch)](https://pytorch.org/docs/stable/notes/amp_examples.html) - examples
  of using mixed precision in PyTorch
- ["Mixed precision training"
  (fast.ai)](https://docs.fast.ai/callback.fp16.html) - this one is another good
  explanation of how mixed precision training works in theory

## Benchmarks

- [MultiPL-E](https://nuprl.github.io/MultiPL-E/)
- [OpenAI's HumanEval](https://github.com/openai/human-eval/)
- [BIG-bench](https://github.com/google/BIG-bench)
- [BIG-Bench-Hard](https://github.com/suzgunmirac/BIG-Bench-Hard)

## Courses / books
- [fast.ai](https://course.fast.ai/)
- [Andrej Karpathy's Neural Networks:
  Zero-to-Hero](https://karpathy.ai/zero-to-hero.html)
- [MIT Deep Learning Book](https://github.com/janishar/mit-deep-learning-book-pdf)

## Blogs / websites / Wikipedia
- [Illustrated Transformer - Jay
  Alammar](https://jalammar.github.io/illustrated-transformer/)
- [All you need to know about Attention and Transformers - Arjun
  Sarkar](https://towardsdatascience.com/all-you-need-to-know-about-attention-and-transformers-in-depth-understanding-part-1-552f0b41d021)
- [Positional encoding - Hunter
  Phillips](https://medium.com/@hunter-j-phillips/positional-encoding-7a93db4109e6)
- [A tutorial on LLM - Haifeng
  Li](https://medium.com/@haifengl/a-tutorial-to-llm-f78dd4e82efc) - Meta
  information about dataset collection, fine-tuning, transfer learning, types of
  applications where LLM's can be used etc.
- [Why multi-head self attention works: math, intuitions and 10+1 hidden
  insights - AI Summer](https://theaisummer.com/self-attention/) - Really good
  article
